#include <malloc.h>
#include <stdio.h>

#include "libpdf/pdfbool.h"
#include "libpdf/pdfdictionary.h"
#include "libpdf/pdffloat.h"
#include "libpdf/pdfint.h"
#include "libpdf/pdfliteral.h"
/*#include "libpdf/pdfname.h"*/
#include "libpdf/pdfstring.h"

PDF_Dict_Entry PDF_Dict_Entry_new(PDF_Name key, PDF_Object value)
{
	PDF_Dict_Entry retval;
	
	if (key == NULL)
		return NULL;

	if (value == NULL)
		return NULL;
	

	retval = (PDF_Dict_Entry) PDF_Object_create(sizeof(struct PDF_dict_entry), PDF_DICT_ENTRY, PDF_CBOOL_FALSE);
	if (retval == NULL)
		return NULL;

	retval->key = key;
	retval->value = value;

	return retval;
}

void PDF_Dict_Entry_delete(PDF_Dict_Entry entry)
{

	PDF_Object_delete((PDF_Object) ((PDF_Dict_Entry)entry)->key);
	PDF_Object_delete(((PDF_Dict_Entry)entry)->value);
}
	 
int PDF_Dict_Entry_write(PDF_Dict_Entry obj, FILE*  ostream, int index,
						 PDF_CBool put_indirect, PDF_Xref_Table xref,
						 PDF_Queue indqueue)
{
	int numwritten;

	index = PDF_Object_write((PDF_Object) PDF_Dict_Entry_get_key(obj), ostream,
							 index, put_indirect, xref, indqueue);
	if (index < 0)
		return -1;
	
	numwritten = fprintf(ostream, " ");
	if (numwritten < 0)
		return -1;
	index += numwritten;
	
	index = PDF_Object_write(PDF_Dict_Entry_get_value(obj), ostream, index,
							 PDF_CBOOL_FALSE, xref, indqueue);
	if (index < 0)
		return -1;

	numwritten = fprintf(ostream, "\n");
	if (numwritten <= 0)
		return -1;
	index += numwritten;
	
	return index;
}

PDF_Name PDF_Dict_Entry_get_key(PDF_Dict_Entry entry)
{
	if (entry == NULL)
		return NULL;
	
	return entry->key;
}

PDF_Object PDF_Dict_Entry_get_value(PDF_Dict_Entry entry)
{
	if (entry == NULL)
		return NULL;
	
	return ((PDF_Dict_Entry) entry)->value;
}

void PDF_Dict_Entry_set_value(PDF_Dict_Entry entry, PDF_Object value)
{
	entry->value = value;
}

PDF_Dictionary PDF_Dictionary_new()
{
	return PDF_Queue_new("<< ", " ", ">>\n");
}

PDF_CBool PDF_Dictionary_add_entry(PDF_Dictionary dict, PDF_CString key, PDF_Object value)
{
	PDF_Dict_Entry new_entry;

	new_entry = PDF_Dict_Entry_new(PDF_Literal_new(key), value);
	if (new_entry == NULL)
		return PDF_CBOOL_FALSE;
	
	return PDF_Queue_append((PDF_Queue) dict, (PDF_Object) new_entry);
}

/* This returns a pointer to the entry itself, not to its data.  It's
   done this way to make modification of dictionary entries easier */
PDF_Object PDF_Dictionary_get_entry(PDF_Dictionary dict, PDF_CString key)
{

	PDF_Queue_Node current;

	/* look through the queue for an entry matching the key */
	for (current = PDF_Queue_get_first((PDF_Queue) dict); (current != NULL) &&
		   (strcmp(PDF_Literal_get_value(PDF_Dict_Entry_get_key((PDF_Dict_Entry) PDF_Queue_Node_get_data(current))),
				   key) != 0); current = PDF_Queue_Node_get_next(current)) /* empty */;

	if (current == NULL)
		return NULL;
	
	return PDF_Dict_Entry_get_value((PDF_Dict_Entry) PDF_Queue_Node_get_data(current));
}

int PDF_Dictionary_write(PDF_Dictionary dict, FILE *ostream, int index,
						 PDF_CBool put_indirect, PDF_Xref_Table xref,
						 PDF_Queue indqueue)
{
	return PDF_Queue_write((PDF_Queue) dict, ostream, index, put_indirect,
						   xref, indqueue);
}

