#include <malloc.h>

#include "libpdf/pdfdocument.h"
/*#include "libpdf/pdfname.h"*/
#include "libpdf/pdfliteral.h"

#include "libpdf/pdfxref.h"

/* Note:  the ProcSet entry is documented by Adobe as ``obsolescent''
   (see note at the end of section 8.1).  But Acrobat Reader version 4
   depends on it to generate correct PostScript when printing the
   page.

   I'm going to go ahead and define all the procsets whether I need
   them or not; this will make PS files a bit bigger than necessary,
   but should do no harm. a more efficient implementation would always
   define /PDF, but only define the others as needed.  Of course, that
   would require replacing my short-cut of using a PDF_Literal to
   define the array with a queue.  */
PDF_Document PDF_Document_new(float width, float height)
{
	PDF_Dictionary retval;
	PDF_Dictionary pages;
	PDF_Array mediabox;
	PDF_Dictionary resources;
	PDF_Dictionary fonts;

	retval = NULL;
	pages = NULL;
	mediabox = NULL;
	resources = NULL;
	fonts = NULL;

	if (((retval = PDF_Dictionary_new()) == NULL) ||
		((pages = PDF_Dictionary_new()) == NULL) ||
		((mediabox = PDF_Array_new()) == NULL) ||
		((resources = PDF_Dictionary_new()) == NULL) ||
		((fonts = PDF_Dictionary_new()) == NULL) ||

		(PDF_Array_append(mediabox, (PDF_Object) PDF_Int_new(0)) == PDF_CBOOL_FALSE) ||
		(PDF_Array_append(mediabox, (PDF_Object) PDF_Int_new(0)) == PDF_CBOOL_FALSE) ||
		(PDF_Array_append(mediabox, (PDF_Object) PDF_Float_new(width)) == PDF_CBOOL_FALSE) ||
		(PDF_Array_append(mediabox, (PDF_Object) PDF_Float_new(height)) == PDF_CBOOL_FALSE) ||

		(PDF_Dictionary_add_entry(pages, "/Type", (PDF_Object) PDF_Name_new("/Pages")) == PDF_CBOOL_FALSE) ||
		(PDF_Dictionary_add_entry(pages, "/Kids", (PDF_Object) PDF_Array_new()) == PDF_CBOOL_FALSE) ||
		(PDF_Dictionary_add_entry(pages, "/Count", (PDF_Object) PDF_Int_new(0)) == PDF_CBOOL_FALSE) ||
		(PDF_Dictionary_add_entry(pages, "/MediaBox", (PDF_Object) mediabox) == PDF_CBOOL_FALSE) ||
		(PDF_Dictionary_add_entry(pages, "/Resources", (PDF_Object) resources) == PDF_CBOOL_FALSE) ||

		(PDF_Dictionary_add_entry(resources, "/Font", (PDF_Object) fonts) == PDF_CBOOL_FALSE) ||
		(PDF_Dictionary_add_entry(resources, "/ProcSet", (PDF_Object) PDF_Literal_new("[/PDF /Text /ImageB /ImageC /ImageI]")) == PDF_CBOOL_FALSE) ||
		(PDF_Dictionary_add_entry(resources, "/XObject", (PDF_Object) PDF_Dictionary_new()) == PDF_CBOOL_FALSE) ||
		
		(PDF_Dictionary_add_entry(retval, "/Type", (PDF_Object) PDF_Literal_new("/Catalog")) == PDF_CBOOL_FALSE) ||
		(PDF_Dictionary_add_entry(retval, "/Pages", (PDF_Object) pages) == PDF_CBOOL_FALSE))

	  {
		
		if (retval != NULL) free(retval);
		if (pages != NULL) free(pages);
		if (fonts != NULL) free(fonts);
		
		return NULL;
	}

	PDF_Object_set_indirect((PDF_Object) pages, PDF_CBOOL_TRUE);
	PDF_Object_set_indirect((PDF_Object) resources, PDF_CBOOL_TRUE);
	PDF_Object_set_indirect((PDF_Object) retval, PDF_CBOOL_TRUE);
	
	return (PDF_Document) retval;
}

PDF_CBool PDF_Document_add_Page(PDF_Document doc, PDF_Page page)
{
	PDF_Dictionary pages;
	PDF_Array kids;
	PDF_Int page_count;
	
	/* Get the array of pages from the document's page node */
	pages = (PDF_Dictionary) PDF_Dictionary_get_entry((PDF_Dictionary) doc, "/Pages");
	if (pages == NULL)
		return PDF_CBOOL_FALSE;
	
	kids = (PDF_Array) PDF_Dictionary_get_entry(pages, "/Kids");
	if (kids == NULL)
		return PDF_CBOOL_FALSE;

	/* Put the new page in the kids array, mark the pages node as its
	   parent, and update the page count */
	PDF_Array_append(kids, (PDF_Object) page);
	PDF_Dictionary_add_entry((PDF_Dictionary) page, "/Parent", (PDF_Object) pages);

	page_count = (PDF_Int) PDF_Dictionary_get_entry(pages, "/Count");
	PDF_Int_set_value(page_count, PDF_Int_get_value(page_count) + 1);
	

	return PDF_CBOOL_TRUE;
}
											   
int PDF_Document_write(PDF_Document doc, FILE *ostream)
{
	int index, xrefpos, numwritten;
	PDF_Xref_Table xref;
	PDF_Queue objqueue;
	PDF_Dictionary trailerdict;

	xref = PDF_Xref_Table_new();
	if (xref == NULL)
		return 0;

	objqueue = PDF_Queue_new("", "", "");

	if (objqueue == NULL) {
		free(xref);
		return 0;
	}

	trailerdict = PDF_Dictionary_new();
	
	if (PDF_Queue_append(objqueue, (PDF_Object) doc) == PDF_CBOOL_FALSE) {
		free(xref);
		free(objqueue);
		return 0;
	}

	index = fprintf(ostream, "%%PDF-1.3\n%%%c%c%c%c\n",
					'a' | 0x80, 'b' | 0x80, 'c' | 0x80, 'd' | 0x80);
	if (index < 0)
		return -1;

	/* This can't be PDF_Object_write, as its recursive calls turn of
	   put_indirect and we don't end up actually writing any objects */
	index = PDF_Queue_write(objqueue, ostream, index, PDF_CBOOL_TRUE,
							xref, objqueue);
	if (index < 0)
		return -1;
	
	xrefpos = index;
	index = PDF_Xref_Table_write(xref, ostream, index, PDF_CBOOL_FALSE,
								 xref, objqueue);
	if (index < 0)
		return -1;

	PDF_Dictionary_add_entry(trailerdict, "/Size",
							 (PDF_Object) PDF_Int_new(xref->maxused+1));
	PDF_Dictionary_add_entry(trailerdict, "/Root", (PDF_Object) doc);

	numwritten = fprintf(ostream, "trailer\n");
	if (numwritten < 0)
		return -1;
	index += numwritten;
	
	index = PDF_Dictionary_write(trailerdict, ostream, index, PDF_CBOOL_FALSE,
							   xref, objqueue);
	if (index < 0)
		return -1;
	
	numwritten = fprintf(ostream, "startxref\n");
	if (numwritten < 0)
		return -1;
	index += numwritten;
	
	numwritten = fprintf(ostream, "%d\n", xrefpos);
	if (numwritten < 0)
		return -1;
	index += numwritten;
	
	numwritten = fprintf(ostream, "%%%%EOF\n");
	if (numwritten < 0)
		return -1;
	index += numwritten;
	
	return index;
}

	
