#include <malloc.h>
#include <stdarg.h>

#include "libpdf/pdfmisc.h"

#include "libpdf/pdfpage.h"
#include "libpdf/pdfgraphics.h"

PDF_Graphics PDF_Graphics_new(PDF_Graphics graphics)
{
	PDF_Queue retval;
	
	retval = PDF_Queue_new("q\n" ,"", "Q\n");
	PDF_Graphics_append(graphics, (PDF_Object) retval);

	return (PDF_Graphics) retval;
}

PDF_CBool PDF_Graphics_append(PDF_Graphics graphics, PDF_Object obj)
{
	return PDF_Queue_append((PDF_Queue) graphics, obj);
}

PDF_CBool PDF_Graphics_append_op(PDF_Graphics graphics, PDF_CString format, ...)
{
	char contentbuf[PDF_SMALLBUFSIZ];
	va_list arg_list;
	
	va_start (arg_list, format);
	vsnprintf(contentbuf, PDF_SMALLBUFSIZ, format, arg_list);
	va_end(arg_list);

	return PDF_Graphics_append(graphics, (PDF_Object) PDF_Literal_new(contentbuf));
}

/* graphics state operators */

PDF_CBool PDF_Graphics_concat_matrix(PDF_Graphics obj, float a, float b, float c, float d, float e, float f)
{
	return PDF_Queue_append_op((PDF_Queue) obj, "%f %f %f %f %f %f cm\n", a, b, c, d, e, f);
}

PDF_CBool PDF_Graphics_translate(PDF_Graphics obj, float x, float y)
{
	return PDF_Graphics_concat_matrix(obj, 1.0, 0.0, 0.0, 1.0, x, y);
}

PDF_CBool PDF_Graphics_rotate(PDF_Graphics obj, float theta)
{
	/* need to look this up */
	return PDF_CBOOL_FALSE;
}

PDF_CBool PDF_Graphics_scale(PDF_Graphics obj, float x, float y)
{
	return PDF_Graphics_concat_matrix(obj, x, 0.0, 0.0, y, 0.0, 0.0);
}

PDF_CBool PDF_Graphics_line_width(PDF_Graphics obj, float width)
{
	return PDF_Queue_append_op(obj, "%f w\n", width);
}
	
PDF_CBool PDF_Graphics_line_cap(PDF_Graphics obj, PDF_CapStyle style)
{
	return PDF_Queue_append_op(obj, "%d J\n", style);
}
	 
PDF_CBool PDF_Graphics_line_join(PDF_Graphics obj, PDF_JoinStyle style)
{
	return PDF_Queue_append_op(obj, "%d j\n", style);
}
	 
PDF_CBool PDF_Graphics_mitre_limit(PDF_Graphics obj, float limit)
{
	return PDF_Queue_append_op(obj, "%f M\n", limit);
}

PDF_CBool PDF_Graphics_dash(PDF_Graphics obj, float dasharray[], float phase)
{
	PDF_Array dashes;
	int i;
	
	dashes = PDF_Array_new();
	for (i = 0; dasharray[i] != 0; dasharray++)
		PDF_Array_append(dashes, (PDF_Object) PDF_Float_new(dasharray[i]));

	PDF_Queue_append(obj, (PDF_Object) dashes);
	
	return PDF_Queue_append_op((PDF_Queue) obj, "%f d\n", phase);
}

PDF_CBool PDF_Graphics_intent(PDF_Graphics obj, PDF_ColorIntent intent)
{
	static PDF_CString PDF_Intent_Strings[] = {
	  "AbsoluteColorimetric",
	  "RelativeColorimetric",
	  "Saturation",
	  "Perceptual",
	  NULL
	};

	/* really ought to derive this from the array or the enum */
	if (intent > 3)
		return PDF_CBOOL_FALSE;
	
	return PDF_Queue_append_op(obj, "%s ri\n", PDF_Intent_Strings[intent]);
}

PDF_CBool PDF_Graphics_flatness(PDF_Graphics obj, float flatness)
{
	if ((flatness < 0) || (flatness > 100))
		return PDF_CBOOL_FALSE;
	
	return PDF_Queue_append_op(obj, "%f i\n", flatness);
}

/* yuck.  this will take more work than I want to devote to it today.
   Basically, I need to create a graphics state dictionary type, and
   hook it up much the same way I did fonts....  Won't use a string dictname
PDF_CBool PDF_Graphics_dictionary(PDF_Object stream, PDF_String dictName); */
