#include <stdint.h>
#include <string.h>

#include "libpdf/pdfmisc.h"

#include "libpdf/pdfdictionary.h"
#include "libpdf/pdfdocument.h"
#include "libpdf/pdfimage.h"
#include "libpdf/pdfliteral.h"
#include "libpdf/pdfqueue.h"
#include "libpdf/pdfstream.h"

static images=0;

PDF_Image PDF_Image_new_pgm(PDF_Document doc, gray** inbuf, int cols, int rows, int maxval)
{
	PDF_Stream imageXobject;
	char namebuf[PDF_SMALLBUFSIZ];
	
	int row, col, shift;
	uint8_t *streambuf, *streambufptr;

	/* shift to account for maxval */
	if (maxval < 0x100)
		shift = 0;
	else if (maxval < 0x10000)
		shift = 8;
	else
		shift = 24;
	
	/* copy the array of gray into an array of unsigned chars */
	streambuf = malloc(rows*cols);
	streambufptr = streambuf;
	for (row = 0; row < rows; row++)
		for (col = 0; col < cols; col++)
			*(streambufptr++) = inbuf[row][col] >> shift;

	/* create the x object and define its parameters */
	imageXobject = PDF_Stream_new();
	PDF_Stream_set_param(imageXobject, "/Type", (PDF_Object) PDF_Literal_new("/XObject"));
	PDF_Stream_set_param(imageXobject, "/Subtype", (PDF_Object) PDF_Literal_new("/Image"));
	PDF_Stream_set_param(imageXobject, "/Width", (PDF_Object) PDF_Int_new(cols));
	PDF_Stream_set_param(imageXobject, "/Height", (PDF_Object) PDF_Int_new(rows));
	PDF_Stream_set_param(imageXobject, "/ColorSpace", (PDF_Object) PDF_Literal_new("/DeviceGray"));
	PDF_Stream_set_param(imageXobject, "/BitsPerComponent", (PDF_Object) PDF_Int_new(8));

	/* hook up the image */
	PDF_Queue_append(PDF_Stream_get_content(imageXobject), (PDF_Object) PDF_Literal_connect(streambuf, rows*cols));

	/* Put the X object in the resource dictionary */
	snprintf(namebuf, PDF_SMALLBUFSIZ, "/Image%d", images);

	PDF_Dictionary_add_entry((PDF_Dictionary) PDF_Dictionary_get_entry((PDF_Dictionary) PDF_Dictionary_get_entry((PDF_Dictionary) PDF_Dictionary_get_entry(doc, "/Pages"), "/Resources"), "/XObject"), namebuf, (PDF_Object) imageXobject);

	/* create a string that will draw the image, and return it to the user */
	snprintf(namebuf, PDF_SMALLBUFSIZ, "/Image%d Do\n", images++);
	return (PDF_Image) PDF_Literal_new(namebuf);
}

// pbm is too smart:  when it reads a file, it wants to expand the bits.  I need
// to read it packed, so I need to read it in here instead of passing in a pbm image
PDF_Image PDF_Image_new_pbm_file(PDF_Document doc, FILE *imagefile)
{
	PDF_Stream imageXobject;
	char namebuf[PDF_SMALLBUFSIZ];

	int rows, cols, format;
	int row, col;
	uint8_t *streambuf, *streambufptr;

	pbm_readpbminit(imagefile, &cols, &rows, &format);
	streambuf = malloc(rows*((cols + 7)/8));

	// Need to read in each row, then invert the bits as pbm's idea of black and white is backwards from pdf's.
	// I suspect this can be cured by using a custom color space of some sort...
	for (row = 0; row < rows; row++) {
		pbm_readpbmrow_packed(imagefile, &streambuf[row*pbm_packed_bytes(cols)], cols, format);

		for (col = 0; col < pbm_packed_bytes(cols); col++)
			streambuf[row*pbm_packed_bytes(cols) + col] = ~streambuf[row*pbm_packed_bytes(cols) + col];
	}

	/* create the x object and define its parameters */
	imageXobject = PDF_Stream_new();
	PDF_Stream_set_param(imageXobject, "/Type", (PDF_Object) PDF_Literal_new("/XObject"));
	PDF_Stream_set_param(imageXobject, "/Subtype", (PDF_Object) PDF_Literal_new("/Image"));
	PDF_Stream_set_param(imageXobject, "/Width", (PDF_Object) PDF_Int_new(cols));
	PDF_Stream_set_param(imageXobject, "/Height", (PDF_Object) PDF_Int_new(rows));
	PDF_Stream_set_param(imageXobject, "/ColorSpace", (PDF_Object) PDF_Literal_new("/DeviceGray"));
	PDF_Stream_set_param(imageXobject, "/BitsPerComponent", (PDF_Object) PDF_Int_new(1));

	/* hook up the image */
	PDF_Queue_append(PDF_Stream_get_content(imageXobject), (PDF_Object) PDF_Literal_connect(streambuf, rows*((cols+7)/8)));

	/* Put the X object in the resource dictionary */
	snprintf(namebuf, PDF_SMALLBUFSIZ, "/Image%d", images);

	PDF_Dictionary_add_entry((PDF_Dictionary) PDF_Dictionary_get_entry((PDF_Dictionary) PDF_Dictionary_get_entry((PDF_Dictionary) PDF_Dictionary_get_entry(doc, "/Pages"), "/Resources"), "/XObject"), namebuf, (PDF_Object) imageXobject);

	/* create a string that will draw the image, and return it to the user */
	snprintf(namebuf, PDF_SMALLBUFSIZ, "/Image%d Do\n", images++);
	return (PDF_Image) PDF_Literal_new(namebuf);
}
