#include "libpdf/pdfpath.h"

/* path construction */
PDF_CBool PDF_Path_move(PDF_Path obj, float x, float y)
{
	return PDF_Queue_append_op(obj, "%f %f m\n", x, y);
}

PDF_CBool PDF_Path_line(PDF_Path obj, float x, float y)
{
	return PDF_Queue_append_op(obj, "%f %f l\n", x, y);
}

/* there are three flavors of curve operators; we'll select which one
   the user wants through the controlpoints parameter */
 PDF_CBool PDF_Path_curve(PDF_Path obj, float x1, float y1, float x2, float y2,
					  float x3, float y3, PDF_ControlPoints controlpoints)
{
	switch (controlpoints) {
	  case PDF_CONTROL_EXPLICIT:
		return PDF_Queue_append_op(obj, "%f %f %f %f %f %f c\n",
								   x1, y1, x2, y2, x3, y3);

	  case PDF_CONTROL_FIRST:
		return PDF_Queue_append_op(obj, "%f %f %f %f v\n",
								   x2, y2, x3, y3);

	  case PDF_CONTROL_LAST:
		return PDF_Queue_append_op(obj, "%f %f %f %f y\n",
								   x1, y1, x3, y3);

	  default:
		return PDF_CBOOL_FALSE;
	}
}

PDF_CBool PDF_Path_rectangle(PDF_Path obj, float x, float y, float width, float height)
{
	return PDF_Queue_append_op(obj, "%f %f %f %f re\n", x, y, width, height);
}

/* Path painting */
/* I've tossed the path-close operator (h) in here because it
   logically fits the function's API, even though it doesn't actually
   paint */
PDF_CBool PDF_Path_paint(PDF_Path obj, PDF_CBool closepath, PDF_CBool stroke, PDF_CBool fill, PDF_FillType filltype)
{
	char contentbuf[PDF_SMALLBUFSIZ];
	static char command[8] = {'n', 'f', 's', 'B', 'h', 'f', 'S', 'b'};
	int i, index;

	i = 0;
	index = 4*closepath + 2*stroke + fill;
	contentbuf[i++] = command[index];
	if (fill && (filltype == PDF_FILL_EVENODD))
		contentbuf[i++] = '*';
	contentbuf[i++] = '\n';
	contentbuf[i++] = 0;

	return PDF_Queue_append_op(obj, contentbuf);
}
		
/* Clipping */
PDF_CBool PDF_Path_clip(PDF_Path obj, PDF_FillType filltype)
{
	char contentbuf[PDF_SMALLBUFSIZ];
	int i;

	i = 0;
	contentbuf[i++] = 'W';
	if (filltype == PDF_FILL_EVENODD)
		contentbuf[i++] = '*';
	contentbuf[i++] = '\n';
	contentbuf[i++] = 0;

	return PDF_Queue_append_op(obj, contentbuf);
}
