/* path construction.  This can't be handled as neatly as most
   objects, as there are two different operators that can be used to
   start a path and three drawing operators to finish it.  Also, two
   clipping operators can precede the drawing operators; this means if
   I wanted to handle paths analogously to graphics states or text
   objects, I'd need to define twelve types of path objects.  For now,
   I think we'll let the user worry about it.
*/
#ifndef PATH_H
#define PATH_H

typedef enum {PDF_FILL_EVENODD, PDF_FILL_WINDING} PDF_FillType;
typedef enum {PDF_CONTROL_EXPLICIT, PDF_CONTROL_FIRST, PDF_CONTROL_LAST} PDF_ControlPoints;

#include "libpdf/pdfgraphics.h"
#include "libpdf/pdfpage.h"
#include "libpdf/pdfqueue.h"

typedef PDF_Queue PDF_Path;

/* paths are attached to graphics objects, as defined by graphics.h */
PDF_Path PDF_Path_new(PDF_Graphics obj);

/* path construction */
PDF_CBool PDF_Path_move(PDF_Path obj, float x, float y);
PDF_CBool PDF_Path_line(PDF_Path obj, float x, float y);
PDF_CBool PDF_Path_curve(PDF_Path obj, float x1, float y1, float x2, float y2,
					  float x3, float y3, PDF_ControlPoints controlpoints);
PDF_CBool PDF_Path_rectangle(PDF_Path obj, float x, float y, float width, float height);

/* Path painting */
PDF_CBool PDF_Path_paint(PDF_Path obj, PDF_CBool closepath, PDF_CBool stroke, PDF_CBool fill, PDF_FillType filltype);

/* Clipping */
PDF_CBool PDF_Path_clip(PDF_Path obj, PDF_FillType fill);

#endif
