#include <stdio.h>
#include <sys/stat.h>
#include <ft2build.h>
#include FT_FREETYPE_H
#include <libpdf/pdf.h>

#define PT 72.0
#define HT (11.0*PT)
#define WID (8.5*PT)

#define TOP (.25*PT)
#define BOTTOM (.25*PT)
#define LEFT (.5*PT)
#define RIGHT (.25*PT)
#define GAP (.25*PT)

#define BOXHT ((HT - TOP - BOTTOM - BARSIZE - (3*GAP))/4.0)
#define BOXWID (WID - LEFT - RIGHT)

#define BARSIZE GAP
#define BARWID (1.0)
#define BARCHARS 5.0
#define BARSCALE (16.0*100.0*BARWID/(BARSIZE))
#define BARX LEFT
#define BARY (BOTTOM + BOXHT)

#define TITLESIZE 12.0
#define TITLEGAP (.2*PT)
#define TITLEX (LEFT + TITLEGAP)
#define TITLEY (BOTTOM + BOXHT - TITLESIZE)

char *titles[4] = {
  "Gold Award (state qualifier)",
  "Silver Award (state qualifier)",
  "Bronze Award (state qualifier)",
  "Pewter Award (not state qualifier)",
};

main() {
	PDF_Document doc;
	PDF_Page page;
	PDF_Font barfont, titlefont;
	PDF_Graphics boxes;
	PDF_Text bartext, titletext;
	
	char textbuf[128];
	int numwritten;
	int boxnum;
	
	/* create a document */
	doc = PDF_Document_new(WID, HT);

	/* create a page and add it to the document */
	page = PDF_Page_new(doc);

	/* get barcode and Helvetica fonts */
	barfont = PDF_Font_new_embed_file(doc, "Code39.ttf");
	titlefont = PDF_Font_new_standard(doc, PDF_FONT_HELVETICA);

	/* create a graphics object for the boxes */
	boxes = PDF_PageContent_new(page);

	/* create a text object for the barcodes */
	bartext = PDF_Text_new(PDF_PageContent_new(page));

	PDF_Text_set_font(bartext, barfont, BARSIZE);
	PDF_Text_set_scale(bartext, BARSCALE);
	PDF_Text_newline_xy(bartext, BARX, BARY);

	/* create a text object for the Titles */
	titletext = PDF_Text_new(PDF_PageContent_new(page));

	PDF_Text_set_font(titletext, titlefont, TITLESIZE);
	PDF_Text_set_scale(titletext, 100.0);	
	PDF_Text_newline_xy(titletext, TITLEX, TITLEY);
	
	/* Fill in the boxes and barcodes */
	for (boxnum = 3; boxnum >= 0; boxnum--) {
		/* the box */
		PDF_Path_rectangle(boxes, LEFT, BOTTOM + boxnum*(BOXHT+GAP), BOXWID, BOXHT);
		PDF_Path_paint(boxes, PDF_CBOOL_FALSE, PDF_CBOOL_TRUE, PDF_CBOOL_FALSE, PDF_FILL_EVENODD);

		/* the barcode */
		if (boxnum < 3)
			PDF_Text_newline_xy(bartext, 0, BOXHT+GAP);
		sprintf(textbuf, "*C%d%d*", boxnum+1, boxnum+1);
		PDF_Text_string(bartext, textbuf);

		/* the title */
		if (boxnum < 3)
			PDF_Text_newline_xy(titletext, 0, BOXHT+GAP);
		PDF_Text_string(titletext, titles[boxnum]);
	}
	
	/* put document to standard out */
	numwritten = PDF_Document_write(doc, stdout);

	return 0;
}

	
